package smash.appointment.parse;

import java.io.FileInputStream;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.FormulaEvaluator;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;

public class XlsxCalendarProcessor {
	Logger						 logger	= Logger.getLogger(XlsxCalendarProcessor.class);

	private SubjectDao subjectDao;

	public List<AppointmentEntry> processExcel(String filename, Calendar minDate) throws Exception {
		List<AppointmentEntry> result = new ArrayList<AppointmentEntry>();
		InputStream inp = new FileInputStream(filename);
		Workbook workbook = WorkbookFactory.create(inp);
		Iterator<Sheet> sheetIter = workbook.sheetIterator();
		while (sheetIter.hasNext()) {
			Sheet sheet = sheetIter.next();
			String name = sheet.getSheetName().trim();
			if (name.contains("2017")) {
				String monthName = name.split(" ")[0];
				String monthNumber = parseMonth(monthName);
				result.addAll(processSheet(sheet, "2017-" + monthNumber));
			} else {
				logger.debug("Skipping sheet: " + name);
			}
		}
		if (minDate != null) {
			Set<AppointmentEntry> toBeRemoved = new HashSet<>();

			for (AppointmentEntry entry : result) {
				if (isBefore(entry, minDate)) {
					toBeRemoved.add(entry);
				}
			}
			result.removeAll(toBeRemoved);
		}

		return result;
	}

	private static final SimpleDateFormat DATE_FORMATTER = new SimpleDateFormat("yyyy-MM-dd");

	private boolean isBefore(AppointmentEntry entry, Calendar minDate) {
		String entryDate = entry.getDay();
		String beforeDate = DATE_FORMATTER.format(minDate.getTime());
		if (entryDate.compareTo(beforeDate) < 0) {
			return true;
		} else {
			return false;
		}
	}

	int[]	dayColumns						 = new int[] { 3, 4, 5, 6, 7 };

	int[]	weekStartRows					 = new int[] { 5, 23, 41, 59, 77, 95 };

	int		hourColum							 = 0;
	int		dayOfMonthRowOffset		 = 0;
	int		calendarRowStartOffset = 3;
	int		calendarRowEndOffset	 = 18;

	private List<AppointmentEntry> processSheet(Sheet sheet, String string) {
		FormulaEvaluator evaluator = sheet.getWorkbook().getCreationHelper().createFormulaEvaluator();

		List<AppointmentEntry> result = new ArrayList<AppointmentEntry>();

		CellParser parser = new CellParser();
		parser.setSubjectDao(subjectDao);

		for (int weekOffset : weekStartRows) {
			Row weekRow = sheet.getRow(weekOffset + dayOfMonthRowOffset);
			for (int dayColumnOffset : dayColumns) {
				Cell dayCell = weekRow.getCell(dayColumnOffset);
				String dayOfMonth = "0";
				evaluator.evaluateInCell(dayCell);
				if (dayCell.getCellTypeEnum().equals(CellType.NUMERIC)) {
					dayOfMonth = ((int) dayCell.getNumericCellValue()) + "";
				} else {
					dayOfMonth = dayCell.getStringCellValue();
				}
				if (dayOfMonth.length() == 1) {
					dayOfMonth = "0" + dayOfMonth;
				}
				if (!dayOfMonth.equals("00") && !dayOfMonth.trim().isEmpty()) {
					String day = string + "-" + dayOfMonth;

					String hour = "08:00";
					for (int hourOffset = calendarRowStartOffset; hourOffset < calendarRowEndOffset; hourOffset++) {
						Row hourRow = sheet.getRow(weekOffset + hourOffset);

						Cell hourCell = hourRow.getCell(hourColum);
						if (hourCell.getCellTypeEnum().equals(CellType.NUMERIC)) {

							SimpleDateFormat formatTime = new SimpleDateFormat("HH:mm");
							String hourString = formatTime.format(hourCell.getDateCellValue());

							if (isHour(hourString)) {
								hour = hourString;
							}
						}

						Cell queryCell = hourRow.getCell(dayColumnOffset);
						if (queryCell != null) {
							if (queryCell.getCellTypeEnum().equals(CellType.STRING)) {
								String query = queryCell.getStringCellValue();

								if (query != null && !query.isEmpty()) {
									AppointmentEntry entry = parser.parseAppointment(query, hour);
									entry.setDay(day);

									result.add(entry);
								}
							} else if (!queryCell.getCellTypeEnum().equals(CellType.BLANK)) {
								logger
										.warn("Skipping cell: " + queryCell.getAddress() + ", " + queryCell.getSheet().getSheetName() + ", type: " + queryCell.getCellTypeEnum());
							}
						}
					}
				}
			}

		}
		return result;
	}

	Pattern timePattern = Pattern.compile("^[0-9][0-9]\\:[0-9][0-9]");

	private boolean isHour(String hourString) {
		Matcher matcher = timePattern.matcher(hourString);
		return matcher.find();
	}

	private String parseMonth(String monthName) throws ParseException {
		Date date = new SimpleDateFormat("MMMM", Locale.ENGLISH).parse(monthName);
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		String result = (cal.get(Calendar.MONTH) + 1) + "";
		if (result.length() == 1) {
			result = "0" + result;
		}
		return result;
	}

	/**
	 * @return the subjectDao
	 * @see #subjectDao
	 */
	public SubjectDao getSubjectDao() {
		return subjectDao;
	}

	/**
	 * @param subjectDao
	 *          the subjectDao to set
	 * @see #subjectDao
	 */
	public void setSubjectDao(SubjectDao subjectDao) {
		this.subjectDao = subjectDao;
	}
}
