from django.contrib.auth.models import User
from django.test import TestCase
from django.test import Client

from functions import get_test_location, create_subject, create_worker
from web.forms import SubjectAddForm, get_new_screening_number
from web.models.constants import SEX_CHOICES_MALE, SUBJECT_TYPE_CHOICES_CONTROL


class SubjectAddFormTests(TestCase):
    def setUp(self):
        location = get_test_location()
        self.sample_data = {'first_name': 'name',
                            'last_name': 'name',
                            'sex': SEX_CHOICES_MALE,
                            'type': SUBJECT_TYPE_CHOICES_CONTROL,
                            'default_location': location.id,
                            'screening_number': "123",
                            'country': 'Luxembourg'
                            }
        self.client = Client()
        username = 'piotr'
        password = 'top_secret'
        self.user = User.objects.create_user(
            username=username, email='jacob@bla', password=password)
        self.worker = create_worker(self.user, True)

        self.client.login(username=username, password=password)

    def test_validation(self):
        form = SubjectAddForm(data=self.sample_data, user=self.user)
        form.is_valid()
        self.assertTrue(form.is_valid())

    def test_invalid(self):
        form_data = self.sample_data
        form_data['screening_number'] = "123"

        form = SubjectAddForm(data=form_data, user=self.user)
        form.is_valid()
        self.assertTrue(form.is_valid())
        form.save()

        form2 = SubjectAddForm(data=form_data, user=self.user)
        validation_status = form2.is_valid()
        self.assertFalse(validation_status)
        self.assertTrue("screening_number" in form2.errors)

    def test_invalid_2(self):
        form_data = self.sample_data
        form_data['nd_number'] = "123"

        form = SubjectAddForm(data=form_data, user=self.user)
        form.is_valid()
        self.assertTrue(form.is_valid())
        form.save()

        form_data['screening_number'] = "2"
        form2 = SubjectAddForm(data=form_data, user=self.user)
        validation_status = form2.is_valid()
        self.assertFalse(validation_status)
        self.assertTrue("nd_number" in form2.errors)

    def test_invalid_3(self):
        form_data = self.sample_data
        form_data['screening_number'] = ""

        form = SubjectAddForm(data=form_data, user=self.user)
        validation_status = form.is_valid()
        self.assertFalse(validation_status)
        self.assertTrue("screening_number" in form.errors)

    def test_get_new_screening_number(self):
        prefix = "X-"
        subject = create_subject()
        subject.screening_number = prefix + "300"
        subject.save()

        new_screening_number = get_new_screening_number(prefix)
        self.assertEqual(prefix + "301", new_screening_number)

    def test_get_new_screening_number_2(self):
        prefix = "X-"
        subject = create_subject()
        subject.screening_number = "L-1111; " + prefix + "300"
        subject.save()

        new_screening_number = get_new_screening_number(prefix)
        self.assertEqual(prefix + "301", new_screening_number)

    def test_get_new_screening_number_3(self):
        prefix = "X-"
        subject = create_subject()
        subject.screening_number = prefix + "300" + "; L-1111"
        subject.save()

        new_screening_number = get_new_screening_number(prefix)
        self.assertEqual(prefix + "301", new_screening_number)

    def test_get_new_screening_number_3(self):
        prefix = "X-"
        subject = create_subject()
        subject.screening_number = "P-1112; " + prefix + "300" + "; L-1111"
        subject.save()

        new_screening_number = get_new_screening_number(prefix)
        self.assertEqual(prefix + "301", new_screening_number)

    def test_get_new_screening_number_4(self):
        prefix = "X-"
        subject = create_subject()
        subject.screening_number = "P-1112; "
        subject.save()

        new_screening_number = get_new_screening_number(prefix)
        self.assertEqual(prefix + "001", new_screening_number)

    def test_get_new_screening_number_5(self):
        prefix = "X-"
        subject = create_subject()
        subject.screening_number = prefix + "100"
        subject.save()

        subject = create_subject()
        subject.screening_number = prefix + "200"
        subject.save()

        subject = create_subject()
        subject.screening_number = "Y-300"
        subject.save()

        subject = create_subject()
        subject.screening_number = prefix + "20"
        subject.save()

        new_screening_number = get_new_screening_number(prefix)
        self.assertEqual(prefix + "201", new_screening_number)

    def test_get_new_screening_number_6(self):
        prefix = "X-"
        subject = create_subject()
        subject.screening_number = "X-"
        subject.save()

        new_screening_number = get_new_screening_number(prefix)
        self.assertEqual(prefix + "001", new_screening_number)
