# coding=utf-8
from django.contrib import messages
from django.shortcuts import redirect, get_object_or_404

from notifications import get_subjects_with_no_visit, get_subjects_with_reminder
from . import wrap_response
from ..forms import SubjectAddForm, SubjectEditForm, VisitDetailForm, get_prefix_screening_number
from ..models import Subject, Worker

SUBJECT_LIST_GENERIC = "GENERIC"
SUBJECT_LIST_NO_VISIT = "NO_VISIT"
SUBJECT_LIST_REQUIRE_CONTACT = "REQUIRE_CONTACT"

def subjects(request):
    context = {
        'list_type': SUBJECT_LIST_GENERIC,
    }
    return wrap_response(request, 'subjects/index.html', context)


def subject_add(request):
    if request.method == 'POST':
        form = SubjectAddForm(request.POST, request.FILES, user=request.user)
        if form.is_valid():
            form.save()
            messages.add_message(request, messages.SUCCESS, 'Subject created')
            return redirect('web.views.subject_edit', id=form.instance.id)
    else:
        worker = Worker.get_by_user(request.user)
        form = SubjectAddForm(user=request.user, initial={'screening_number': get_prefix_screening_number(worker)})

    return wrap_response(request, 'subjects/add.html', {'form': form})


def subject_no_visits(request):
    context = {
        'list_type': SUBJECT_LIST_NO_VISIT,
    }
    return wrap_response(request, 'subjects/index.html', context)


def subject_require_contact(request):
    context = {
        'list_type': SUBJECT_LIST_REQUIRE_CONTACT,
    }
    return wrap_response(request, 'subjects/index.html', context)


def subject_edit(request, id):
    the_subject = get_object_or_404(Subject, id=id)
    contact_attempts = the_subject.contactattempt_set.order_by('-datetime_when').all()
    was_dead = the_subject.dead
    was_resigned = the_subject.resigned
    if request.method == 'POST':
        form = SubjectEditForm(request.POST, request.FILES, instance=the_subject, was_dead=was_dead,
                               was_resigned=was_resigned)
        if form.is_valid():
            form.save()
            # check if subject was marked as dead or resigned
            if form.cleaned_data['dead'] and not was_dead:
                the_subject.mark_as_dead()
            if form.cleaned_data['resigned'] and not was_resigned:
                the_subject.mark_as_resigned()
            return redirect('web.views.subjects')
    else:
        form = SubjectEditForm(instance=the_subject, was_dead=was_dead, was_resigned=was_resigned)

    return wrap_response(request, 'subjects/edit.html', {
        'form': form,
        'subject': the_subject,
        'contact_attempts': contact_attempts
    })


def subject_visit_details(request, id):
    subjects = get_object_or_404(Subject, id=id)
    visits = subjects.visit_set.all()
    visits_data = []
    for visit in visits:
        appointments = visit.appointment_set.all()
        finished = visit.is_finished
        visit_id = visit.id
        visit_title = visit.follow_up_title()
        visit_form = VisitDetailForm(instance=visit)
        visits_data.append((visit_form, appointments, finished, visit_id, visit_title))

    return wrap_response(request, 'subjects/visitdetails.html', {'display': visits_data, "id": id})
